/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.internal.javascript.debug.ui.launching;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.ui.AbstractLaunchConfigurationTab;
import org.eclipse.e4.languages.internal.javascript.debug.launching.Constants;
import org.eclipse.e4.languages.internal.javascript.debug.launching.ILaunchConstants;
import org.eclipse.e4.languages.internal.javascript.debug.ui.ISharedImages;
import org.eclipse.e4.languages.internal.javascript.debug.ui.JSDIImageRegistry;
import org.eclipse.e4.languages.internal.javascript.debug.ui.SWTFactory;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;

/**
 * Tab to allow connection information to be specified
 * 
 * @since 0.9
 */
public class JavaScriptConnectTab extends AbstractLaunchConfigurationTab {

	Text hosttext = null, porttext = null;
	
	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#createControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createControl(Composite parent) {
		Composite comp = SWTFactory.createComposite(parent, 2, 1, GridData.FILL_HORIZONTAL);
		SWTFactory.createLabel(comp, "&Host:", 1);
		this.hosttext = SWTFactory.createSingleText(comp, 1);
		GridData gd = (GridData) this.hosttext.getLayoutData();
		gd.grabExcessHorizontalSpace = true;
		this.hosttext.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updateLaunchConfigurationDialog();
			}
		});
		SWTFactory.createLabel(comp, "&Port:", 1);
		this.porttext = SWTFactory.createSingleText(comp, 1);
		gd = (GridData) this.porttext.getLayoutData();
		gd.grabExcessHorizontalSpace = true;
		this.porttext.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				updateLaunchConfigurationDialog();
			}
		});
		setControl(comp);
		//TODO set help
		/*PlatformUI.getWorkbench().getHelpSystem().setHelp(comp, "id");*/
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#isValid(org.eclipse.debug.core.ILaunchConfiguration)
	 */
	public boolean isValid(ILaunchConfiguration launchConfig) {
		int port = -1;
		try {
			port = Integer.parseInt(this.porttext.getText().trim());
		}
		catch(NumberFormatException nfe) {
			setErrorMessage("You must specify a number for the port");
			return false;
		}
		if(port < 0) {
			setErrorMessage("You must specify a positive integer as a port number");
			return false;
		}
		if(Constants.EMPTY_STRING.equals(this.hosttext.getText().trim())) {
			setErrorMessage("You must specify a host name to connect to");
			return false;
		}
		setErrorMessage(null);
		return true;
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#getName()
	 */
	public String getName() {
		return "Connect";
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.AbstractLaunchConfigurationTab#getImage()
	 */
	public Image getImage() {
		return JSDIImageRegistry.getSharedImage(ISharedImages.IMG_CONNECT);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#initializeFrom(org.eclipse.debug.core.ILaunchConfiguration)
	 */
	public void initializeFrom(ILaunchConfiguration configuration) {
		try {
			String value = configuration.getAttribute(ILaunchConstants.HOST, (String)null);
			if(value != null) {
				this.hosttext.setText(value);
			}
			else {
				this.hosttext.setText(Constants.EMPTY_STRING);
			}
			value = configuration.getAttribute(ILaunchConstants.PORT, (String)null);
			if(value != null) {
				this.porttext.setText(value);
			}
			else {
				this.porttext.setText(Constants.EMPTY_STRING);
			}
		}
		catch(CoreException ce) {
			//do nothing
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#performApply(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void performApply(ILaunchConfigurationWorkingCopy configuration) {
		String value = this.hosttext.getText().trim();
		if(!Constants.EMPTY_STRING.equals(value)) {
			configuration.setAttribute(ILaunchConstants.HOST, value);
		}
		value = this.porttext.getText().trim();
		if(!Constants.EMPTY_STRING.equals(value)) {
			configuration.setAttribute(ILaunchConstants.PORT, value);
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.debug.ui.ILaunchConfigurationTab#setDefaults(org.eclipse.debug.core.ILaunchConfigurationWorkingCopy)
	 */
	public void setDefaults(ILaunchConfigurationWorkingCopy configuration) {}
}
