/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.javascript.debug.model;

import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.model.IDebugTarget;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.e4.languages.javascript.jsdi.VirtualMachine;

/**
 * The Javascript debug model
 * 
 * @since 1.0
 */
public class JSDIDebugModel {

	/**
	 * Debug model identifier
	 */
	public static final String MODEL_ID = "org.eclipse.e4.languages.javascript.debug.model"; //$NON-NLS-1$

	/**
	 * Returns a new {@link JSDIDebugTarget} that can disconnect and terminate by default
	 * 
	 * @param vm
	 *            the underlying {@link VirtualMachine}
	 * @param name
	 *            the name for the target
	 * @param process
	 *            the underlying process
	 * @param launch
	 *            the launch that spawned the target
	 * @return a new {@link JSDIDebugTarget}
	 */
	public static IDebugTarget newJSDIDebugTarget(VirtualMachine vm, String name, IProcess process, ILaunch launch) {
		return newJSDIDebugtarget(vm, process, launch, name, true, true);
	}

	/**
	 * Returns a new {@link JSDIDebugTarget}
	 * 
	 * @param vm
	 *            the {@link VirtualMachine} to connect to
	 * @param process
	 *            the underlying {@link IProcess}
	 * @param launch
	 *            the launch that spawned this target
	 * @param name
	 *            the name for the target or <code>null</code>
	 * @param canterminate
	 *            if the target will support terminating
	 * @param candisconnect
	 *            if the target will support disconnecting
	 * @return a new {@link JSDIDebugTarget}
	 */
	static IDebugTarget newJSDIDebugtarget(VirtualMachine vm, IProcess process, ILaunch launch, String name, boolean canterminate, boolean candisconnect) {
		return new JSDIDebugTarget(vm, process, launch, name, canterminate, candisconnect);
	}
}
